/******************************************************************************
*      TrayIcon demonstrates how to add, modify and delete tray icons.        *
*                     1996, 1997, 1998 by Mark Gamber                         *
*******************************************************************************
* By using this program or any part of it, the user assumes full respon-      *
* -sibility for it's use and may not hold the author liable for any loss or   *
* damage. If you cannot accept this restriction, the program may not be used  *
* and must be destroyed immediately.                                          *
******************************************************************************/

#include "windows.h"
#include "resource.h"

// === Function Prototypes ====================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL ModifyTrayIcon( HWND, UINT, UINT, DWORD, int );

// === Global Variables =======================================================

HINSTANCE hInst;
HWND hMainWnd;
int g_iIcon;                       //  Tracks which icon is currently displayed

// === Program Entry Point ====================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
					  int nShow )
{
	MSG msg;


	hInst = hInstance;                               //  Create main dialog box
	hMainWnd = CreateDialog( hInst, MAKEINTRESOURCE( 10000 ), NULL,
							 MainDlgProc );
	if( ! hMainWnd )
		return( FALSE );                          //  If dialog fails, exit app

	ShowWindow( hMainWnd, nShow );                          //  Show the window
	UpdateWindow( hMainWnd );

	while( GetMessage( &msg, NULL, 0, 0 ) )              //  Start message loop
	{
		if( IsDialogMessage( hMainWnd, &msg ) )     //  If dialog message, just
			continue;                            //  loop back for next message
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( FALSE );
}

// === Main Window (Dialog) Procedure =========================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
			g_iIcon = 1;                                //  Add icon #1 to tray
			ModifyTrayIcon( hDlg, WM_COMMAND, 1000, NIM_ADD, g_iIcon );
			break;


		case WM_DESTROY:
			PostQuitMessage( 0 );         //  end program when dialog is closed
			break;


		case WM_COMMAND:
			if( wParam == IDCANCEL )                      //  If cancel pressed
			{                         //  Remove the tray icon and close window
				ModifyTrayIcon( hDlg, WM_COMMAND, 1000, NIM_DELETE, g_iIcon );
				DestroyWindow( hDlg );
			}
			if( wParam == 100 )                   //  If Change Icon is pressed
			{
				if( g_iIcon == 1 )                     //  If icon #1 is in use
					g_iIcon = 2;                          //  Change to icon #2
				else
					g_iIcon = 1;                 //  Otherwise, make it icon #1
				                         //  Modify tray icon to reflect change
				ModifyTrayIcon( hDlg, WM_COMMAND, 1000, NIM_MODIFY, g_iIcon );
				break;
			}
			if( wParam == 1000 )                //  If tray icon was clicked...
			{
				if( lParam == 0x0201 )            //  Ignore all but left click
					MessageBox( hDlg, "Left-clicked icon", "trayicon",
								MB_OK | MB_ICONINFORMATION );
				break;
			}
			break;
	}
	return( FALSE );
}                                                      //  End of MainDlgProc()

// === Handles Icon Stuff =====================================================
// hWnd:   Handle of window to receive notification messages
// msg:    Message to use for notification
// uID:    Notification ID passed in wParam
// dwType: NIM_ constant
// iIcon:  Number of icon in resource file
// ============================================================================

BOOL ModifyTrayIcon( HWND hWnd, UINT msg, UINT uID, DWORD dwType, int iIcon )
{
	NOTIFYICONDATA nid;


	memset( &nid, 0, sizeof(NOTIFYICONDATA) );         //  Initialize structure
	nid.cbSize = sizeof(NOTIFYICONDATA);             //  Plug in structure size
	nid.hWnd = hWnd;                                  //  Plug in passed values
	nid.uID = uID;
	nid.uFlags = NIF_ICON | NIF_MESSAGE;      //  We use icons and notification
	nid.uCallbackMessage = msg;                    //  Load icon from resources
	nid.hIcon = LoadImage( hInst, MAKEINTRESOURCE( iIcon ),
						   IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR );
	Shell_NotifyIcon( dwType, &nid );          //  Update icon using shell call
	return( TRUE );
}

// ============================================================================


